;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x85fb8373ab51101fdece2bc4d658a1c5 . "./code.cg2js/cg-common-control.fasl")
(#xa2818098a942191dacfca643807a483f . "./code.cg2js/cg-clipboard-stack.fasl")
(#x60fe98f373dd51c8f6e9f430d7bf5aba . "./code.cg2js/cg-yes-no-string.fasl")
(#x00bcef0f5e32537b498769b73ddd6180 . "./code.cg2js/cg-clipboard.fasl")
(#x55d226fbec778a19302b84c03bbc6736 . "./code.cg2js/cg-dde-utils.fasl")
(#xf80ea2b47c1496c799bd54d45780dbff . "./code.cg2js/cg-editable-text.fasl")
(#x26979bd9b59c3e577fb712dcf0d43432 . "./code.cg2js/cg-player.fasl")
(#x28342584bc30ac034c68b8e479a4b5e0 . "./code.cg2js/cg-text-or-combo.fasl")
(#xc4540a7677903ed41190d6b6f64aa1a2 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x8df7b8cbdedae381b13cb748db798db6 . "./code.cg2js/cg-graphics-context.fasl")
(#x2ee2f0238a71132505d2683ff74a5858 . "./code.cg2js/cg-choose-printer.fasl")
(#x3eb4c0f731f70617956c58a8790650f5 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xa074f64ed03eee31c3cb23df2b8d46cb . "./code.cg2js/cg-edit-in-place.fasl")
(#x773308822be628d95cced4efff575fce . "./code.cg2js/cg-split-bar.fasl")
(#x944e93c90dfc64c7b0a5feff57363835 . "./code.cg2js/cg-timer.fasl")
(#xb1af6c4b8d2f695e6262ec20ea76d358 . "./code.cg2js/cg-progress-indicator.fasl")
(#xce4197d7e2b6f0cd9b258c0d8bf8aa97 . "./code.cg2js/cg-object-editor.fasl")
(#x206a9f80b0b1dedf691c8f1442a1dbbc . "./code.cg2js/cg-fill-texture.fasl")
(#xfc5f4ff2e9179ecfb64b0570c8798f26 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x407badd224cd2feed351da66196e97ff . "./code.cg2js/cg-os-window.fasl")
(#xedce944400246114dd8ee2ad308b25ed . "./code.cg2js/cg-color-dialog.fasl")
(#xd9eb9b79622b4ccd8826902ade5265b5 . "./code.cg2js/cg-combo-box.fasl")
(#xa17841dd0975b38f28c8e5ec516e8300 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5763b87594f53571f2d9275e4c9040c4 . "./code.cg2js/cg-tooltip.fasl")
(#x1c1ad7192fb8d9ad4fc7d7f052df53da . "./code.cg2js/cg-lettered-menu.fasl")
(#xec8a72e52e474a69775ba85e3e02a241 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xfaf3e204eebe5502b268ae91977e1fb9 . "./code.cg2js/cg-pixmap.fasl")
(#x0a7dbca802574c51532a7f152022c4b3 . "./code.cg2js/cg.fasl")
(#xd8cb93e343c7d160ba06fd02d80a6fca . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5424091408a1503420b4da686c1ed23e . "./code.cg2js/cg-template-string.fasl")
(#xaa70975bbcfa40af31edb24bc19a8da8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe27c66b29dd6e6d1c1335a18fc0a3ba0 . "./code.cg2js/cg-check-box.fasl")
(#xf4097798004610e84d2f950d04e39ad4 . "./code.cg2js/cg-toggling-widget.fasl")
(#x6160d7485e6b926656153e6c78849121 . "./code.cg2js/cg-radio-button.fasl")
(#xcf75109b2039763f2fc6cedc62c4e8ac . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf1eaa84e4d10d1ad3333b42d1fdcdc1d . "./code.cg2js/cg-scaling-stream.fasl")
(#x67613280fedfba3843f16978502ba226 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xb7d3fef7b9c5a737b55b893a69107d80 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf033ae19c061f949c616bc2698431b0d . "./code.cg2js/cg-grid-widget.fasl")
(#xece668481741fc1b6acbd33c6af2578e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x57402c39b0659b4cfc41a71cbfeab5af . "./code.cg2js/cg-item-list.fasl")
(#x133c789fd07c54ba35d61396629683a6 . "./code.cg2js/cg-paren-matching.fasl")
(#xe00ca1c2b630c847d4adeef89bef3e50 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x201bbffc6a30fca28080b345faaa517f . "./code.cg2js/cg-html-dialog.fasl")
(#x6303a6724a48e69ce1f3581f37f4f240 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9fa3c68af7fce2c71f9e93f3f67aa9df . "./code.cg2js/cg-project-window.fasl")
(#x69f82050c0403294a5b0779f0087ea45 . "./code.cg2js/cg-toolbar.fasl")
(#xf976a5b6c13a475d9c3fea659cd834fe . "./code.cg2js/cg-caret.fasl")
(#x8b74d28dd4ae497f964a66869e27becc . "./code.cg2js/cg-scroll-bar.fasl")
(#xb0be9f2ef28b57ff9b39ec8a27153abd . "./code.cg2js/cg-web-browser.fasl")
(#xafd848693910a9bbd48da6bdf7022c46 . "./code.cg2js/cg-dropping-outline.fasl")
(#x48e33ea3cd59ce606b44e692068132be . "./code.cg2js/cg-font-dialog.fasl")
(#xe1b66a09b970e7cda7e90f8d13a33deb . "./code.cg2js/cg-icon-pixmap.fasl")
(#x22b2d394e0c10bde8409d936db3cffc6 . "./code.cg2js/cg-yes-no-list.fasl")
(#x0f67fbd924efc8dc62c52a52a654ec7a . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xfedcdcb4b45927b266ca28aadd70458d . "./code.cg2js/cg-lamp.fasl")
(#x618e153dd40da7fef057b679d78fc299 . "./code.cg2js/cg-drawable.fasl")
(#x07fa12abf7f0485e8069f7140d52f02b . "./code.cg2js/cg-choice-list.fasl")
(#x259d5693ed24e3a232bd884f107a8372 . "./code.cg2js/cg-curve.fasl")
(#x7e0e4ac7a0a4f52c7a116c0a9bfefb54 . "./code.cg2js/cg-base.fasl")
(#xbab61cf59e90b5e4f6eadb83c417745e . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb6b3c6a3f744b31c1b41f0cf4b52bc49 . "./code.cg2js/cg-printing.fasl")
(#x66c3d5d6e4fe584c1edd957418ff537d . "./code.cg2js/cg-chart-or-plot.fasl")
(#xacdb861d49703c0e63fdf07e3124c430 . "./code.cg2js/cg-hotspot.fasl")
(#x7d458f601ca014ac97511dcb8d9ec283 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x13a642a7dc036231b0ddf7a49a40214f . "./code.cg2js/cg-utility-dialog.fasl")
(#x9333fd981bd28305c4678ab2838249f8 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xae2f9ef38412c7da0f0b69cd7cc2b63f . "./code.cg2js/cg-comtab.fasl")
(#x25eae694c5a38f323b8dd4a620d7ddcc . "./code.cg2js/cg-os-widget.fasl")
(#x1e28f0e435ecfc7962c9e7a792f8fab3 . "./code.cg2js/cg-icon.fasl")
(#x74d0c8f472d0e651d00b201d53859932 . "./code.cg2js/cg-dev.fasl")
(#x4324323baf3cb0419e0831701162b3a0 . "./code.cg2js/cg-calendar.fasl")
(#x83b4cf34ffe5e04197e1973a244e7219 . "./code.cg2js/cg-group-box.fasl")
(#x1455b08700e3c94383a927490fbc7102 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7945d511e03d216ce52fbf1f180ae3e2 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x712a8cc80464787b0ecb0470b1a0d9cc . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4bfa5db2b7c1440aa4ef2a33b11e852d . "./code.cg2js/cg-nodes.fasl")
(#x23c5d442c337d5b73827adb660348691 . "./code.cg2js/cg-intersect.fasl")
(#x7c4a8cf370dbc2912df0b246390d8730 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x1c9dd1dea2e14861671c3ad634e05304 . "./code.cg2js/cg-class-support.fasl")
(#xb35937f5b287c64844d9127f84f923a9 . "./code.cg2js/cg-dialog-item.fasl")
(#x1c49584c97ea069829527455613ea014 . "./code.cg2js/cg-plot-widget.fasl")
(#x657709c0b83578e6470592acd70c9be4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xde5f0c8f58c944223cc5a5492c9554dd . "./code.cg2js/cg-up-down-control.fasl")
(#x83b7c5b32f32c10f790c04b90b316fa3 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x1015e9b0deb548464281ae29d4a0370e . "./code.cg2js/cg-status-bar.fasl")
(#xce14a0ba2ad3029dc62d08229abef5d4 . "./code.cg2js/cg-class-grid.fasl")
(#x4f548805f17608622992a98898938c6c . "./code.cg2js/cg-wrap-string.fasl")
(#xb59a463a7ffdadb3f553f78b0c5436cb . "./code.cg2js/cg-layout.fasl")
(#x856191f6cb3457f588d2194189ad4fe0 . "./code.cg2js/cg-lisp-widget.fasl")
(#x726eb27b6adb2f1e40704eded7fa3008 . "./code.cg2js/cg-button.fasl")
(#x31ae3f838f9e9d193cda70f254492d43 . "./code.cg2js/cg-string-dialog.fasl")
(#x819cde3f6e0bf07fefcd2d4083f8b9b2 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8596c7f9934e9e19e78b0508524fb44b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7b8dda24319deccc6a30e608b0992b6e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x262a504422aa29d3c9408628b327ab0a . "./code.cg2js/cg-static-text.fasl")
(#xdfc8cfb7b64401c056155284a1dd1579 . "./code.cg2js/cg-outline.fasl")
(#x88f87105aa2cf15a48747d2f509f48e2 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc04abc213d54984ab415d26306d77c56 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x4e08191ea4c574047ca895b8f8ee8b41 . "./code.cg2js/cg-picture-widget.fasl")
(#x360fae7a5f2af142252e4ff66dbe1121 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xf9577e88ee499fcbc229670ec36d8140 . "./code.cg2js/cg-chart-widget.fasl")
(#x29a63e71daac16f9aa157a32896d77a8 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xaa10dedc0b78619159ab8a7d7a21fcaa . "./code.cg2js/cg-lisp-text.fasl")
(#xa3c935ac223f25d54863dcff9f13ef62 . "./code.cg2js/cg-palette.fasl")
(#x3f414efb2b57ebb76cc97b7b209d26af . "./code.cg2js/cg-property.fasl")
(#xd30583141df4caa0a401bfab257c2409 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe330c32100b4c39ace2593503fec772f . "./code.cg2js/cg-file-dialog.fasl")
(#x4bf7c4937e4acc989d0f9aa3e3b5e7ea . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x79fd4b7107ecbf02bab8bebc609a542f . "./code.cg2js/cg-get-pixmap.fasl")
(#x56c8429f49f3f63a9488f7e5e024bedf . "./code.cg2js/cg-tray.fasl")
(#x8411a21c0e1f998a9f3db616c837174a . "./code.cg2js/cg-acache.fasl")
(#xf61d5746a0290a1074a6bab183206efa . "./code.cg2js/cg-message-dialog.fasl")
(#xca8e8cf6120575a0cf08979643a1f75d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfd2121a71fc63b132544202b71f36cc7 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe838b4be11cff9a40f25a8a30dbb8b83 . "./code.cg2js/cg-tab-control.fasl")
(#x0c2b876043b73c9517c0839cdb943faa . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x229d5bc99b6c93ffc9fe6ab30395331a . "./code.cg2js/cg-menu-tooltip.fasl")
(#x780f8fdd6802e58dac49101787323435 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x4a2453d0142cb9dac17e522e72ad03d9 . "./code.cg2js/cg-menu.fasl")
(#x2a0cb103332775e607052fe0abd85b17 . "./code.cg2js/cg-get-position.fasl")
(#x1fec03c152b8e77ab9725a8d1f77413b . "./code.cg2js/cg-trackbar.fasl")
(#x9f522b5670768bb0f015137efe01deeb . "./code.cg2js/cg-selected-object.fasl")
(#x22f35fc8b77ef193d2b7324f4aecfad7 . "./code.cg2js/cg-list-view.fasl")
(#x5f28d18c0e41e4ec7e5a04e9db44e44c . "./code.cg2js/cg-html-widget.fasl")
(#xa3547d7719c671874a57b570d456ea78 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x9c24e8e872a6bdea92611a7c620a2f23 . "./code.cg2js/cg-text-widget.fasl")
(#xe2bbfce3855b5589e76f6ae8b463ed9a . "./code.idejs/ide-base.fasl")
(#x27e17896815715ef1361723377ac3694 . "./code.idejs/ide-process-browser.fasl")
(#xfefb7ec84525273f2d71fa6b322f0c43 . "./code.idejs/ide-patch-dialog.fasl")
(#xe3f43b0f13b3098d6c87c8a12aff8656 . "./code.idejs/ide-keyshort.fasl")
(#xe4c5a1ec21520085ae50d80ee6198684 . "./code.idejs/ide-window-list.fasl")
(#xd503970c93811d3deee9d123b7ee71a3 . "./code.idejs/ide.fasl")
(#xd3245f587531d3f1547898905bc01fb9 . "./code.idejs/ide-editor.fasl")
(#xc573abb5e4d556e660c5dd16f8c306d7 . "./code.idejs/ide-clipboard.fasl")
(#xb8246f517df9c0c98ffc890e586bbae5 . "./code.idejs/ide-trace.fasl")
(#x53404da2cb3cfd4ba46de70163777df6 . "./code.idejs/ide-class-browser.fasl")
(#x1d13e342458e653bf810a04d3af55cf1 . "./code.idejs/ide-grapher.fasl")
(#xbbe7ca21fb57155d573e9877cd36839a . "./code.idejs/ide-apropos.fasl")
(#xe071fcdcfcd3ed9b3c772ec208075bc1 . "./code.idejs/ide-navigator.fasl")
(#xcd4b80a37b3f55dafb567400544b261f . "./code.idejs/ide-find-definitions.fasl")
(#x17ff77af0c3ac7fbca2cddaa9501bd66 . "./code.idejs/ide-project.fasl")
(#x905bee8d23eb17225a103d8760dcb798 . "./code.idejs/ide-class-grapher.fasl")
(#xd316d72c7ddfadb7d73fa8e71d4e3039 . "./code.idejs/ide-acache.fasl")
(#x8552ef0b641aa4fa4c15c7803c490e69 . "./code.idejs/ide-find-in-files.fasl")
(#xf6417e02152640329d20a951affc37fc . "./code.idejs/ide-inspector.fasl")
(#x9d8e6125d3a9270887e413874542799a . "./code.idejs/ide-intro.fasl")
(#x859c7091cb89c725de524b020c4039f7 . "./code.idejs/ide-package-list.fasl")
(#x60dc7192967d9324fda57d2fba45b82c . "./code.idejs/ide-dev.fasl")
(#x911322a23335e41bf674b2923d9d17c0 . "./code.idejs/ide-form.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
